@@ -132,13 +132,16 @@ end
 bootstrap_steps = Dir.glob(File.expand_path('chake/bootstrap/*.sh', File.dirname(__FILE__))).sort
 
 desc 'Executed before bootstrapping'
-task :bootstrap_common
+task :bootstrap_common => :connect_common
 
 desc 'Executed before uploading'
-task :upload_common
+task :upload_common => :connect_common
 
 desc 'Executed before uploading'
-task :converge_common
+task :converge_common => :connect_common
+
+desc 'Executed before connecting to any host'
+task :connect_common
 
 $nodes.each do |node|
 
@@ -223,24 +226,24 @@ $nodes.each do |node|
   end
 
   desc 'apply <recipe> on #{hostname}'
-  task "apply:#{hostname}", [:recipe] => [:recipe_input] do |task, args|
+  task "apply:#{hostname}", [:recipe] => [:recipe_input, :connect_common] do |task, args|
     chef_logging = Rake.application.options.silent && '-l fatal' || ''
     node.run_as_root "chef-solo -c #{node.path}/#{chef_config} #{chef_logging} -j #{node.path}/#{$chake_tmpdir}/#{hostname}.json --override-runlist recipe[#{$recipe_to_apply}]"
   end
   task "apply:#{hostname}" => converge_dependencies
 
   desc "run a command on #{hostname}"
-  task "run:#{hostname}", [:command] => [:run_input] do
+  task "run:#{hostname}", [:command] => [:run_input, :connect_common] do
     node.run($cmd_to_run)
   end
 
   desc "Logs in to a shell on #{hostname}"
-  task "login:#{hostname}" do
+  task "login:#{hostname}" => :connect_common do
     node.run_shell
   end
 
   desc 'checks connectivity and setup on all nodes'
-  task "check:#{hostname}" do
+  task "check:#{hostname}" => :connect_common do
     node.run('sudo true')
   end
 
@@ -289,23 +292,25 @@ task :recipe_input, :recipe do |task,args|
 end
 
 desc "upload to all nodes"
-task :upload => $nodes.map { |node| "upload:#{node.hostname}" }
+multitask :upload => $nodes.map { |node| "upload:#{node.hostname}" }
 
 desc "bootstrap all nodes"
-task :bootstrap => $nodes.map { |node| "bootstrap:#{node.hostname}" }
+multitask :bootstrap => $nodes.map { |node| "bootstrap:#{node.hostname}" }
 
 desc "converge all nodes (default)"
-task "converge" => $nodes.map { |node| "converge:#{node.hostname}" }
+multitask "converge" => $nodes.map { |node| "converge:#{node.hostname}" }
 
 desc "Apply <recipe> on all nodes"
-task "apply", [:recipe] => $nodes.map { |node| "apply:#{node.hostname}" }
+multitask "apply", [:recipe] => $nodes.map { |node| "apply:#{node.hostname}" }
 
 desc "run <command> on all nodes"
-task :run, [:command] => $nodes.map { |node| "run:#{node.hostname}" }
+multitask :run, [:command] => $nodes.map { |node| "run:#{node.hostname}" }
 
 task :default => :converge
 
 desc 'checks connectivity and setup on all nodes'
-task :check => ($nodes.map { |node| "check:#{node.hostname}" }) do
+multitask :check => ($nodes.map { |node| "check:#{node.hostname}" }) do
   puts "✓ all hosts OK"
+  puts "  - ssh connection works"
+  puts "  - password-less sudo works"
 end
